from PyQt5.QtWidgets import QLabel, QColorDialog
from PyQt5.QtGui import QColor
from PyQt5.QtCore import pyqtSignal


class ClickableLabel(QLabel):

    clicked = pyqtSignal()

    def __init__(self, color: QColor):
        super(QLabel, self).__init__()
        self.setGeometry(100, 100, 200, 60)

        self.color = color
        self.redrawLabel()

        self.clicked.connect(self.changeColor)

    def mousePressEvent(self, e):
        self.clicked.emit()

    def changeColor(self):
        print(f"Changing {self.color.name()}")
        color = QColorDialog.getColor(
            initial=self.color,
            title="Select Color",
            options=QColorDialog.ShowAlphaChannel,
        )
        self.color = color if color.isValid() else self.color
        print(f"to {self.color.name()}")

        # setting option
        # dialog.setOption(QColorDialog.ShowAlphaChannel)

        # executing the dialog
        # dialog.exec_()
        self.redrawLabel()

    def redrawLabel(self):

        self.setStyleSheet(
            "QLabel {"
            f"background-color : {self.color.name()};"
            "border : 1px solid black;"
            "}"
        )



import os
from pathlib import Path

t_base = Path(os.path.dirname(__file__))

basedir = t_base.resolve()
# print(basedir)  # RePhraser\src\rephraser

# get the project folder name two levels up (e.g. "RePhraser")
project_dir = t_base.parents[1]
# print(project_dir)  # RePhraser



from PyQt5.QtCore import Qt
from PyQt5.QtGui import QColor

# todo: use the Qt namespace colors


class AuthorEntry:
    def __init__(
        self,
        author_name: str = "Default",
        foreground: QColor = QColor("#ffffff"),
        background: QColor = QColor("#a6a6a6"),
        weight: int = 100,
        italic: bool = True,
        href: str = "www.example.com",
    ):
        self.author_name = author_name
        self.foreground = foreground if type(foreground) == QColor else QColor(foreground)
        self.background = background if type(background) == QColor else QColor(background)
        self.weight = weight if type(weight) == int else 100
        self.italic = italic
        self.href = href

    def getProperties(self, include_name=False) -> dict:
        prop = {
            "foreground": self.foreground,
            "background": self.background,
            "italic": self.italic,
            "weight": self.weight,
            "href": self.href,
        }

        if include_name:
            prop["author_name"] = self.author_name

        return prop

    def getStyleSheet(self) -> str:
        signature = f"""
            color: {self.foreground.name()};
            background: {self.background.name()};
            font-weight: {self.weight * 8};
            font-style: {"italic" if self.italic else ""};
        """
        return signature



from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *

from rephraser.lib.helper import *
from rephraser.lib.Stores import store
from rephraser.lib.qt_helper import HLine
from rephraser.lib.Logger import Logger
from rephraser.lib.DarkPallete import enable_dark_titlebar

import math

IMAGE_EXTENSIONS = [".jpg", ".png", ".bmp"]


class PasteFromAuthorDialog(QDialog):

    author_selected = pyqtSignal(str)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.parent_ = parent
        self.setWindowTitle("Paste as Author")

        mainlayout = QVBoxLayout()

        author_cont = QHBoxLayout()

        self.author_cmbx = QComboBox()
        self.fillComboBox()

        author_cont.addWidget(QLabel("Existing Author: "))
        author_cont.addWidget(self.author_cmbx)

        self.saveAuthor_btn = QPushButton("Ok")
        self.saveAuthor_btn.clicked.connect(lambda: self.fin())

        self.newAuthor_btn = QPushButton("Add as new Author")
        self.newAuthor_btn.clicked.connect(self.addNewAuthor)

        hline = HLine(parent=self)

        mainlayout.addLayout(author_cont)
        mainlayout.addWidget(self.saveAuthor_btn)
        mainlayout.addWidget(hline)
        mainlayout.addWidget(self.newAuthor_btn)

        self.setLayout(mainlayout)

    def addNewAuthor(self):
        self.parent_.parent_.author_table.addAuthor("")
        self.fillComboBox()

    def fillComboBox(self):
        self.author_cmbx.clear()
        self.author_cmbx.addItems(store.author_dictionary.keys())

    def fin(self):
        self.done(QDialog.Accepted)
        self.author_selected.emit(self.author_cmbx.currentText())


class TextEdit(QTextEdit):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

        self.parent_ = parent

        self.setMinimumWidth(580)
        
        # Make the caret thicker
        self.setCursorWidth(3)

        self.parent_ = parent
        self.textIsSelected = False
        self.dropped_text = None

        self.textCharFormat = QTextCharFormat()
        self.defaultCharFormat = QTextCharFormat()  # reset format
        font = QFont("Lexend", 12)
        self.defaultCharFormat.setFont(font)

        self.images = {}
        self.DPM = math.floor(1 * 39.37)

        # print(self.editor.verticalScrollBar().styleSheet())
        self.verticalScrollBar().setStyle(
            QCommonStyle()
        )  # to make the transparency work

        # self.setDisabled(True)

    def dropEvent(self, event):
        m = event.mimeData()
        if m.hasText():
            self.dropped_text = m.text()

        super().dropEvent(event)

    def canInsertFromMimeData(self, source):
        if source.hasImage():
            # print("I CAN INSERT IMAGE")
            # print(self.parent().path)
            # if not os.path.exist(self.parent().path):
            #     self.parent().file_save()

            # # todo: emit a signal indicating that we need to save first
            # if os.path.exists(self.parent().path):
            #     return True

            # return False
            return True
        else:
            return super(TextEdit, self).canInsertFromMimeData(source)

    def insertFromMimeData(self, source):
        print("INSERT FROM MIME DATA")
        cursor = self.textCursor()
        document = self.document()

        if source.hasImage():
            print("I CAN INSERT IMAGE")
            print(self.parent_.path)
            if self.parent_.path is None:
                QMessageBox.information(
                    self,
                    "Save Required",
                    "Please save the file before inserting images.",
                )
                self.parent_.file_save()

            # recheck after displaying save dialog
            if (self.parent_.path is None) or (not os.path.exists(self.parent_.path)):
                return
                # return True

            print("INSERTING IMAGE")
            image = source.imageData()

            uuid = self.addImageResource(image)
            # fragment = QTextDocumentFragment.fromHtml(
            #     f"<img src='{source.text()}' height='{height}' width='{width}'></img>"
            # )

            # cursor.insertFragment(fragment)
            self.setAlignment(Qt.AlignCenter)
            cursor.insertImage(uuid)
            # cursor.insertImage(uuid)
            return

        elif source.hasText():
            selectedTextIsBeingDragged = self.dropped_text == source.text()

            if not selectedTextIsBeingDragged:
                te = PasteFromAuthorDialog(parent=self)

                enable_dark_titlebar(te)
                te.author_selected.connect(self.setTextCharFormat)
                if te.exec_() == QDialog.Rejected:
                    self.textCharFormat = self.defaultCharFormat

                self.setCharFormatSelection()
                self.textCursor().insertText(source.text(), self.textCharFormat)
                self.removeCharFormatSelection()
            elif selectedTextIsBeingDragged:
                self.textCursor().insertHtml(source.html())

            return

        elif source.hasUrls():
            for u in source.urls():
                file_ext = splitext(str(u.toLocalFile()))
                if u.isLocalFile() and file_ext in IMAGE_EXTENSIONS:
                    image = QImage(u.toLocalFile())
                    # document.addResource(QTextDocument.ImageResource, u, image)
                    # uuid = hexuuid()
                    print("IMAGE FROM FILE")
                    uuid = self.addImageResource(image)
                    cursor.insertImage(uuid)

                else:
                    # If we hit a non-image or non-local URL break the loop and fall out
                    # to the super call & let Qt handle it
                    break

            else:
                # If all were valid images, finish here.
                return

        super().insertFromMimeData(source)

    def keyPressEvent(self, e):
        if e.text().isalnum() or (e.text() == " "):
            # self.removeCharFormatSelection()
            Logger.w("ALPHANUMERIC", Logger.INFO)
            self.textCharFormat = QTextCharFormat(self.defaultCharFormat)
            self.defaultCharFormat.setFontPointSize(self.fontPointSize())
            self.defaultCharFormat.setFont(self.currentFont())
            self.textCursor().insertText(e.text(), self.defaultCharFormat)
            # self.parent_.update_format()
            return

        super().keyPressEvent(e)

    def setCharFormatSelection(self):
        self.textCursor().setCharFormat(self.textCharFormat)

    def removeCharFormatSelection(self):
        self.textCursor().setCharFormat(self.defaultCharFormat)

    def setTextCharFormat(self, authorName):
        prop = store.author_dictionary[authorName]
        Logger.w(prop, Logger.INFO)

        # Create a new format based on current font to preserve font family and size
        self.textCharFormat = QTextCharFormat()
        self.textCharFormat.setFont(self.currentFont())
        self.textCharFormat.setFontPointSize(self.fontPointSize())
        
        # Apply italic formatting
        self.textCharFormat.setFontItalic(prop["italic"])
        
        # Convert weight to Qt font weight system
        # Qt uses: Light=25, Normal=50, DemiBold=63, Bold=75, Black=87
        # Convert from 0-100 scale to Qt scale
        weight_value = prop["weight"]
        if weight_value >= 90:
            qt_weight = QFont.Black  # 87
        elif weight_value >= 70:
            qt_weight = QFont.Bold  # 75
        elif weight_value >= 55:
            qt_weight = QFont.DemiBold  # 63
        elif weight_value >= 35:
            qt_weight = QFont.Normal  # 50
        else:
            qt_weight = QFont.Light  # 25
        
        self.textCharFormat.setFontWeight(qt_weight)
        
        # Apply colors
        self.textCharFormat.setForeground(QColor(prop["foreground"]))
        self.textCharFormat.setBackground(QColor(prop["background"]))

    def resizeEvent(self, e):
        # print(f"{self.document().idealWidth()} : {self.width()}")

        # cur_pos = 0

        # while cur_pos < self.textCursor().position():
        #     found_cursor = self.document().find(self.image_regex, cur_pos)

        #     if found_cursor.position() == -1:
        #         print("NO IMAGE FOUND")
        #         break

        #     text = found_cursor.selectedText()

        #     self.image_regex.indexIn(text)
        #     print(f"ima: {self.image_regex.cap(1)}")
        #     print(f"cur_pos: {cur_pos}")

        #     cur_pos = found_cursor.position()
        # maximumImageWidth = self.width() - 32

        # self.removeUnusedResources()

        # resize pictures to match the max width
        # print("TEXT RESIZE EVENT")
        # print(self.images)
        # for uuid in self.images.keys():
        #     print(uuid)
        #     img = self.images[uuid]
        #     # img = QTextImageFormat()
        #     img.setName(uuid)
        #     editorMaxWidth = self.width() - 32
        #     print(f"before: {img.width()} vs. {editorMaxWidth}")

        #     # if img.width() > editorMaxWidth:
        #     print("resizing")
        #     print(f"before: {img.width()} vs. {editorMaxWidth}")
        #     img.setWidth(editorMaxWidth)
        # img.width()

        super().resizeEvent(e)

    def removeUnusedResources(self):
        uuidToRemove = []

        for uuid in self.images.keys():

            resource = self.document().resource(QTextDocument.ImageResource, QUrl(uuid))

            if resource == None:
                uuidToRemove.append(uuid)
                # del self.images[uuid]
                continue

            # self.addImageResource(self.images[uuid])

        for uuid in uuidToRemove:
            del self.images[uuid]

    def addImageResource(self, ImageResource: QImage, uuid=hexuuid()):
        # cases:
        # pasted image: uuid = generated from hash
        # local file draged: uuid = file path
        if uuid == "":
            raise ValueError("provided 'uuid' is empty")

        print("INSERTING IMAGE")

        maximumImageWidth = self.minimumWidth() - 32
        maximumImageHeight = 400 - 32
        print(maximumImageWidth)
        # if width >= maximumImageWidth:
        # conssider padding
        # factor = maximumImageWidth / widthv <-
        # width *= factor <-
        # height *= factor

        # print(resource.value())
        width = ImageResource.width()

        if width > maximumImageWidth:
            print("SCALING TO WIDTH")
            print(f"{width} : {maximumImageWidth}")
            ImageResource = ImageResource.scaledToWidth(
                math.floor(maximumImageWidth), Qt.SmoothTransformation
            )

        height = ImageResource.height()

        if height > maximumImageHeight:
            print("SCALING TO HEIGHT")
            print(f"{height} : {maximumImageHeight}")
            ImageResource = ImageResource.scaledToHeight(
                math.floor(maximumImageHeight), Qt.SmoothTransformation
            )
        # sc_resource.setDotsPerMeterX(self.DPM);
        # sc_resource.setDotsPerMeterY(self.DPM);

        resourcePath = f"{self.parent_.dir}/{self.parent_.baseName}"

        if not os.path.exists(resourcePath):
            os.mkdir(resourcePath)

        image_name = f"{resourcePath}/{uuid}.png"

        # ImageResource.
        ImageResource.save(image_name)

        # textImageFormat = QTextImageFormat()
        # textImageFormat.setName(image_name)
        # if width > maximumImageWidth:
        #     textImageFormat.setWidth(maximumImageWidth)
        # else:
        #     textImageFormat.setWidth(width)

        self.document().addResource(
            QTextDocument.ImageResource, QUrl(image_name), ImageResource
        )

        self.images[image_name] = ImageResource
        # self.images[uuid] = textImageFormat

        return image_name
        # return uuid

    def setCurrentFont(self, font: QFont):
        print(f"Setting Current Font: {font.family()}")
        # Update the default format with the new font
        self.defaultCharFormat.setFont(font)

        # Get current cursor
        cursor = self.textCursor()

        # Create a format with all font properties
        char_fmt = QTextCharFormat()
        char_fmt.setFont(font)

        if cursor.hasSelection():
            # If text is selected, apply format to selection
            cursor.beginEditBlock()
            cursor.mergeCharFormat(char_fmt)
            cursor.endEditBlock()
            self.setTextCursor(cursor)
        else:
            # For cursor without selection, set format for future typing
            cursor.setCharFormat(char_fmt)
            self.setTextCursor(cursor)

            # Also update document default font for consistency
            self.document().setDefaultFont(font)

        # Call parent implementation for proper handling
        super().setCurrentFont(font)
        self.setFocus()
        #     doc = self.document()
        #     doc_fmt = doc.defaultTextOption()
        #     new_doc_fmt = QTextOption(doc_fmt)
        #     self.document().setDefaultFont(font)

        # # Call the parent implementation for non-selected text
        # super().setCurrentFont(font)
        # self.setFocus()



from PyQt5.QtWidgets import QScrollBar


class ScrollBar(QScrollBar):
    def __init__(self, *args, **kwargs):
        super(QScrollBar, self).__init__(*args, **kwargs)

    def enterEvent(self, e):
        self.setStyleSheet(
            """
        QScrollBar::vertical:hover {
            min-height: 0px;
            border: 0px solid red;
            border-radius: 4px;
            /*width: 8px;*/
            width: 4px;
        }"""
        )

    def leaveEvent(self, e):
        self.setStyleSheet("")



from rephraser import project_dir

from datetime import datetime
import os
import glob


class Logger:
    DEBUG = "Debug"
    INFO = "Info"
    WARNING = "Warn"
    ERROR = "Error"
    CRITICAL = "Crit"

    _file = None
    _initialized = False

    @classmethod
    def _init(cls):
        if not cls._initialized:
            cls.cleanup_old_logs()
            now = datetime.now().strftime("%Y-%m-%d %H-%M-%S")
            logs_dir = project_dir / "logs"
            logs_dir.mkdir(parents=True, exist_ok=True)
            cls._file = open(logs_dir / f"{now}.txt", "x")
            cls._initialized = True

    @classmethod
    def cleanup_old_logs(cls, keep=0):
        logs_dir = project_dir / "logs"
        log_files = sorted(logs_dir.glob("*.txt"), key=os.path.getmtime, reverse=True)
        for old_log in log_files[keep:]:
            try:
                old_log.unlink()
            except Exception:
                pass

    @classmethod
    def w(cls, message, level):
        if not cls._initialized:
            cls._init()
        cls._file.write(f"[{datetime.now().strftime('%H:%M:%S')}][{level}] {message}\n")
        cls._file.flush()

    @classmethod
    def close(cls):
        if cls._file:
            cls._file.close()
            cls._file = None
            cls._initialized = False


Logger._init()



from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtPrintSupport import *

import os
import math

from rephraser.lib.AuthorTable import AuthorTable
from rephraser.lib.ScrollBar import ScrollBar
from rephraser.lib.TextEdit import TextEdit
from rephraser.lib.helper import *
from rephraser.lib.Toolbar import Toolbar
from rephraser.lib.DarkPallete import enable_dark_titlebar
from rephraser.lib.Logger import Logger

from rephraser import basedir
import rephraser.images.images  # Resource file for the icons
from pathlib import Path

floor = math.floor

HTML_EXTENSIONS = [".htm", ".html"]


class MainWindow(QMainWindow):
    changed = False
    dockwidget = None

    def createDock(self):
        dock_widgets = self.findChildren(QDockWidget)
        if self.dockwidget in dock_widgets:
            return

        self.dockwidget = QDockWidget("Change Author")
        self.dockwidget.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
        self.dockwidget.setMaximumWidth(300)
        self.dockwidget.setMinimumWidth(165)

        enable_dark_titlebar(self.dockwidget)

        dock_innerContainer = QWidget()
        self.dockwidget.setWidget(dock_innerContainer)
        dock_layout = QVBoxLayout(dock_innerContainer)

        self.author_table = AuthorTable(parent=self)
        dock_layout.addWidget(self.author_table)

        addAuthor_btn = QPushButton("Add Author")
        addAuthor_btn.clicked.connect(lambda: self.author_table.addAuthor())

        resetFmt_btn = QPushButton("Reset Format")
        resetFmt_btn.clicked.connect(lambda: self.editor.removeCharFormatSelection())

        dock_layout.addWidget(addAuthor_btn)
        dock_layout.addWidget(resetFmt_btn)

        self.addDockWidget(Qt.RightDockWidgetArea, self.dockwidget)

        # ━━━━━━━━━━━━━━━━━━━ Refresh Stylesheet ━━━━━━━━━━━━━━━━━━ #
        self.refresh_btn = QPushButton("Refresh stylesheet")
        self.refresh_btn.clicked.connect(self.refresh_stylesheet)
        dock_layout.addWidget(self.refresh_btn)

        self.dockwidget.closeEvent = lambda event: self.on_dock_close(event)

    def on_dock_close(self, event):
        self.removeDockWidget(self.dockwidget)
        self.dockwidget.deleteLater()
        self.dockwidget = None
        event.accept()

    def __init__(self):
        super().__init__()

        enable_dark_titlebar(self)

        settings = QSettings("Deadbush225", "Rephrase")

        self.setMinimumSize(700, 400)
        self.setWindowIcon(QIcon(os.path.join(basedir, "RePhraser.png")))

        self.path = None
        self.folder = QStandardPaths.writableLocation(QStandardPaths.DocumentsLocation)

        self.layout = QVBoxLayout()

        self.editor = TextEdit(parent=self)
        self.editor.setVerticalScrollBar(ScrollBar(Qt.Vertical))
        self.editor.setTabStopDistance(40)
        self.editor.textChanged.connect(lambda: setattr(self, "changed", True))

        # Setup the QTextEdit editor configuration
        # self.editor.setAutoFormatting(QTextEdit.AutoAll)

        self.layout.addWidget(self.editor)
        # layout.setSpacing(0)
        # layout.setContentsMargins(5, 5, 5, 5)

        container = QWidget()
        container.setLayout(self.layout)

        self.setCentralWidget(container)

        self.status = QStatusBar()
        self.setStatusBar(self.status)

        # Uncomment to disable native menubar on Mac
        # self.menuBar().setNativeMenuBar(False)

        # ━━━━━━━━━━━━━━━━━━━━━━━━ Toolbar ━━━━━━━━━━━━━━━━━━━━━━━━ #
        file_toolbar = Toolbar("File", parent=self)

        # ━━━━━━━━━━━━━━━━━━━━━━ Dock Widget ━━━━━━━━━━━━━━━━━━━━━━ #
        self.createDock()

        # Initialize default font size.
        # self.editor.setFont(font)
        # We need to repeat the size to init the current format.
        self.editor.setFontPointSize(12)

        # ━━━━━━━━━━━━━━━━━━ Initialize Contents ━━━━━━━━━━━━━━━━━━ #
        # self.file_open(
        #     os.path.join(os.path.dirname(os.path.dirname(__file__)), "test.html")
        # )

        # Initialize.
        self.update_title()
        self.setMinimumSize(QSize(780, 510))
        self.show()

    def refresh_stylesheet(self):
        qApp.setStyleSheet("".join(open(os.path.join(basedir, "dark.qss")).readlines()))

    def open_directory(self):
        self.layout.setCurrentWidget(self.editor)

    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()

    def file_open(self, path=""):
        if self.changed:
            res = self.promptUnsavedChanges()
            print(res)
            if res != "Saved" and res != "Discard":
                return

        print(path)
        # print(type(path))
        if not path:
            print("FIND THE FILE")

            path, _ = QFileDialog.getOpenFileName(
                self,
                "Open file",
                self.folder,
                "HTML documents (*.html)",
            )
            if not path:
                return

        print(path)
        # print(type(path))

        try:
            with open(path, "r") as f:
                text = f.read()
                f.close()
                self.folder = os.path.dirname(path)

        except Exception as e:
            self.dialog_critical(str(e))

        else:
            self.path = path
            # Qt will automatically try and guess the format as txt/html

            self.editor.setHtml(text)
            self.changed = False
            # self.update_title()
            self.update_path(path)

    def file_save(self):
        if self.path is None:
            # If we do not have a path, we need to use Save As.
            return self.file_saveas()

        text = (
            self.editor.toHtml()
            if splitext(self.path) in HTML_EXTENSIONS
            else self.editor.toPlainText()
        )

        try:
            with open(self.path, "w") as f:
                f.write(text)
                f.close()
                self.changed = False
                self.update_path(self.path)
                return 1

        except Exception as e:
            self.dialog_critical(str(e))

    def file_saveas(self):
        path, filter_ = QFileDialog.getSaveFileName(
            self,
            "Save file",
            self.folder,
            "HTML documents (*.html)",
        )

        if not path:
            # If dialog is cancelled, will return ''
            return 0
        if Path(path).suffix == "":
            path += ".html"

        text = (
            self.editor.toHtml()
            if splitext(path) in HTML_EXTENSIONS
            else self.editor.toPlainText()
        )

        try:
            with open(path, "w") as f:
                f.write(text)
                f.close()
                self.changed = False
                self.update_path(path)
                return 1

        except Exception as e:
            self.dialog_critical(str(e))

    def file_print(self):
        dlg = QPrintDialog()
        if dlg.exec_():
            self.editor.print_(dlg.printer())

    def update_path(self, path):
        self.path = path
        splitPath = os.path.split(path)
        self.dir = splitPath[0]
        self.fullName = splitPath[1]
        self.baseName = ".".join(self.fullName.split(".")[0:-1])
        self.update_title()

    def update_title(self):
        self.setWindowTitle(
            "%s - RePhraser" % (self.fullName if self.path else "Untitled")
        )

    def edit_toggle_wrap(self):
        self.editor.setLineWrapMode(1 if self.editor.lineWrapMode() == 0 else 0)

    def promptUnsavedChanges(self):
        msg_box = QMessageBox(self)
        msg_box.setIcon(QMessageBox.Question)
        msg_box.setWindowTitle("Unsaved Changes")
        msg_box.setText("You have unsaved changes. Do you want to save them?")
        save_btn = msg_box.addButton("Save", QMessageBox.AcceptRole)
        discard_btn = msg_box.addButton("Discard", QMessageBox.DestructiveRole)
        cancel_btn = msg_box.addButton("Cancel", QMessageBox.RejectRole)
        msg_box.setDefaultButton(save_btn)
        enable_dark_titlebar(msg_box)
        msg_box.exec_()
        clicked_button = msg_box.clickedButton()
        if clicked_button == save_btn:
            res = QMessageBox.Save
        elif clicked_button == discard_btn:
            res = QMessageBox.Discard
        else:
            res = QMessageBox.Cancel

        if res == QMessageBox.Save:
            if self.file_save():
                return "Saved"

        elif res == QMessageBox.Discard:
            return "Discard"

        # elif res == QMessageBox.Cancel:
        return "Cancel"

    def closeEvent(self, e):
        if not self.changed or self.editor.toPlainText() == "":
            return

        res = self.promptUnsavedChanges()

        if res == "Saved" or res == "Discard":
            return
        elif res == "Cancel":
            e.ignore()



import sys
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *

from rephraser.lib.DarkPallete import DarkPalette


class HLine(QFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)
        self.setLineWidth(1)
        # self.setPalette(DarkPalette())



from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

import os

FONT_SIZES = [7, 8, 9, 10, 11, 12, 13, 14, 18, 24, 36, 48, 64, 72, 96, 144, 288]


class Toolbar(QToolBar):
    def reset_toolbar_positions(self):
        """Reset all toolbars to their default positions"""
        main_window = self.parent()

        # First, remove all toolbars from the main window
        main_window.removeToolBar(self)
        main_window.removeToolBar(self.edit_toolbar)
        main_window.removeToolBar(self.format_toolbar)

        # Then add them back in the default positions/areas
        main_window.addToolBar(Qt.TopToolBarArea, self)
        main_window.addToolBar(Qt.TopToolBarArea, self.edit_toolbar)
        main_window.addToolBar(Qt.TopToolBarArea, self.format_toolbar)

        # Make sure they're visible
        self.show()
        self.edit_toolbar.show()
        self.format_toolbar.show()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.parent().addToolBar(self)
        self.setIconSize(QSize(24, 24))
        file_menu = self.parent().menuBar().addMenu("&File")

        open_file_action = QAction(
            QIcon(":/icons/blue-folder-open-document.png"),
            "Open file...",
            self.parent(),
        )
        open_file_action.setStatusTip("Open file")
        open_file_action.setShortcut(QKeySequence.Open)
        open_file_action.triggered.connect(self.parent().file_open)
        file_menu.addAction(open_file_action)
        self.addAction(open_file_action)

        save_file_action = QAction(QIcon(":/icons/disk.png"), "Save", self.parent())
        save_file_action.setStatusTip("Save current page")
        save_file_action.setShortcut(QKeySequence.Save)
        save_file_action.triggered.connect(self.parent().file_save)
        file_menu.addAction(save_file_action)
        self.addAction(save_file_action)

        saveas_file_action = QAction(
            QIcon(":/icons/disk--pencil.png"),
            "Save As...",
            self.parent(),
        )
        saveas_file_action.setStatusTip("Save current page to specified file")
        saveas_file_action.setShortcut(QKeySequence.SaveAs)
        saveas_file_action.triggered.connect(self.parent().file_saveas)
        file_menu.addAction(saveas_file_action)
        self.addAction(saveas_file_action)

        print_action = QAction(
            QIcon(":/icons/printer.png"),
            "Print...",
            self.parent(),
        )
        print_action.setStatusTip("Print current page")
        print_action.triggered.connect(self.parent().file_print)
        file_menu.addAction(print_action)
        self.addAction(print_action)

        self.edit_toolbar = QToolBar("Edit")
        self.edit_toolbar.setIconSize(QSize(24, 24))
        self.parent().addToolBar(self.edit_toolbar)
        edit_menu = self.parent().menuBar().addMenu("&Edit")

        undo_action = QAction(
            QIcon(":/icons/arrow-curve-180-left.png"),
            "Undo",
            self.parent(),
        )
        undo_action.setStatusTip("Undo last change")
        undo_action.triggered.connect(self.parent().editor.undo)
        edit_menu.addAction(undo_action)

        redo_action = QAction(
            QIcon(":/icons/arrow-curve.png"),
            "Redo",
            self.parent(),
        )
        redo_action.setStatusTip("Redo last change")
        redo_action.triggered.connect(self.parent().editor.redo)
        self.edit_toolbar.addAction(redo_action)
        edit_menu.addAction(redo_action)

        edit_menu.addSeparator()

        cut_action = QAction(QIcon(":/icons/scissors.png"), "Cut", self.parent())
        cut_action.setStatusTip("Cut selected text")
        cut_action.setShortcut(QKeySequence.Cut)
        cut_action.triggered.connect(self.parent().editor.cut)
        self.edit_toolbar.addAction(cut_action)
        edit_menu.addAction(cut_action)

        copy_action = QAction(
            QIcon(":/icons/document-copy.png"),
            "Copy",
            self.parent(),
        )
        copy_action.setStatusTip("Copy selected text")
        cut_action.setShortcut(QKeySequence.Copy)
        copy_action.triggered.connect(self.parent().editor.copy)
        self.edit_toolbar.addAction(copy_action)
        edit_menu.addAction(copy_action)

        paste_action = QAction(
            QIcon(":/icons/clipboard-paste-document-text.png"),
            "Paste",
            self.parent(),
        )
        paste_action.setStatusTip("Paste from clipboard")
        cut_action.setShortcut(QKeySequence.Paste)
        paste_action.triggered.connect(self.parent().editor.paste)
        self.edit_toolbar.addAction(paste_action)
        edit_menu.addAction(paste_action)

        select_action = QAction(
            QIcon(":/icons/selection-input.png"),
            "Select all",
            self.parent(),
        )
        select_action.setStatusTip("Select all text")
        cut_action.setShortcut(QKeySequence.SelectAll)
        select_action.triggered.connect(self.parent().editor.selectAll)
        edit_menu.addAction(select_action)

        edit_menu.addSeparator()

        wrap_action = QAction(
            QIcon(":/icons/arrow-continue.png"),
            "Wrap text to window",
            self.parent(),
        )
        wrap_action.setStatusTip("Toggle wrap text to window")
        wrap_action.setCheckable(True)
        wrap_action.setChecked(True)
        wrap_action.triggered.connect(self.parent().edit_toggle_wrap)
        edit_menu.addAction(wrap_action)

        self.format_toolbar = QToolBar("Format")
        self.format_toolbar.setIconSize(QSize(24, 24))
        self.parent().addToolBar(self.format_toolbar)
        format_menu = self.parent().menuBar().addMenu("&Format")

        # We need references to these actions/settings to update as selection changes, so attach to self.parent().
        self.parent().fonts = QFontComboBox()
        self.parent().fonts.currentFontChanged.connect(
            self.parent().editor.setCurrentFont
        )
        font = QFont("Lexend", 12)
        self.parent().fonts.setCurrentFont(font)
        self.parent().editor.setFont(
            font
        )  # needed, since line above doesn't fire "currentFontChanged"

        self.format_toolbar.addWidget(self.parent().fonts)

        self.parent().fontsize = QComboBox()
        self.parent().fontsize.addItems([str(s) for s in FONT_SIZES])

        # Connect to the signal producing the text of the current selection. Convert the string to float
        # and set as the pointsize. We could also use the index + retrieve from FONT_SIZES.
        self.parent().fontsize.currentIndexChanged[str].connect(
            self.update_size
            # lambda s: self.parent().editor.setFontPointSize(float(s))
        )
        self.format_toolbar.addWidget(self.parent().fontsize)

        bold_image = QImage(":/icons/edit-bold.png")
        bold_image.invertPixels()
        bold_pixmap = QPixmap.fromImage(bold_image)

        self.parent().bold_action = QAction(QIcon(bold_pixmap), "Bold", self.parent())
        self.parent().bold_action.setStatusTip("Bold")
        self.parent().bold_action.setShortcut(QKeySequence.Bold)
        self.parent().bold_action.setCheckable(True)
        self.parent().bold_action.toggled.connect(
            lambda x: self.parent().editor.setFontWeight(
                QFont.Bold if x else QFont.Normal
            )
        )
        self.format_toolbar.addAction(self.parent().bold_action)
        format_menu.addAction(self.parent().bold_action)

        italic_image = QImage(":/icons/edit-italic.png")
        italic_image.invertPixels()
        italic_pixmap = QPixmap.fromImage(italic_image)

        self.parent().italic_action = QAction(
            QIcon(italic_pixmap), "Italic", self.parent()
        )
        self.parent().italic_action.setStatusTip("Italic")
        self.parent().italic_action.setShortcut(QKeySequence.Italic)
        self.parent().italic_action.setCheckable(True)
        self.parent().italic_action.toggled.connect(self.parent().editor.setFontItalic)
        self.format_toolbar.addAction(self.parent().italic_action)
        format_menu.addAction(self.parent().italic_action)

        underline_image = QImage(":/icons/edit-underline.png")
        underline_image.invertPixels()
        underline_pixmap = QPixmap.fromImage(underline_image)

        self.parent().underline_action = QAction(
            QIcon(underline_pixmap), "Underline", self.parent()
        )
        self.parent().underline_action.setStatusTip("Underline")
        self.parent().underline_action.setShortcut(QKeySequence.Underline)
        self.parent().underline_action.setCheckable(True)
        self.parent().underline_action.toggled.connect(
            self.parent().editor.setFontUnderline
        )
        self.format_toolbar.addAction(self.parent().underline_action)
        format_menu.addAction(self.parent().underline_action)

        format_menu.addSeparator()

        alignl_image = QImage(":/icons/edit-alignment.png")
        alignl_image.invertPixels()
        alignl_pixmap = QPixmap.fromImage(alignl_image)

        self.parent().alignl_action = QAction(
            QIcon(alignl_pixmap), "Align left", self.parent()
        )
        self.parent().alignl_action.setStatusTip("Align text left")
        self.parent().alignl_action.setCheckable(True)
        self.parent().alignl_action.triggered.connect(
            lambda: self.parent().editor.setAlignment(Qt.AlignLeft)
        )
        self.format_toolbar.addAction(self.parent().alignl_action)
        format_menu.addAction(self.parent().alignl_action)

        alignc_image = QImage(":/icons/edit-alignment-center.png")
        alignc_image.invertPixels()
        alignc_pixmap = QPixmap.fromImage(alignc_image)

        self.parent().alignc_action = QAction(
            QIcon(alignc_pixmap), "Align center", self.parent()
        )
        self.parent().alignc_action.setStatusTip("Align text center")
        self.parent().alignc_action.setCheckable(True)
        self.parent().alignc_action.triggered.connect(
            lambda: self.parent().editor.setAlignment(Qt.AlignCenter)
        )
        self.format_toolbar.addAction(self.parent().alignc_action)
        format_menu.addAction(self.parent().alignc_action)

        alignr_image = QImage(":/icons/edit-alignment-right.png")

        alignr_image.invertPixels()
        alignr_pixmap = QPixmap.fromImage(alignr_image)

        self.parent().alignr_action = QAction(
            QIcon(alignr_pixmap), "Align right", self.parent()
        )
        self.parent().alignr_action.setStatusTip("Align text right")
        self.parent().alignr_action.setCheckable(True)
        self.parent().alignr_action.triggered.connect(
            lambda: self.parent().editor.setAlignment(Qt.AlignRight)
        )
        self.format_toolbar.addAction(self.parent().alignr_action)
        format_menu.addAction(self.parent().alignr_action)

        alignj_image = QImage(":/icons/edit-alignment-justify.png")

        alignj_image.invertPixels()
        alignj_pixmap = QPixmap.fromImage(alignj_image)

        self.parent().alignj_action = QAction(
            QIcon(alignj_pixmap), "Justify", self.parent()
        )
        self.parent().alignj_action.setStatusTip("Justify text")
        self.parent().alignj_action.setCheckable(True)
        self.parent().alignj_action.triggered.connect(
            lambda: self.parent().editor.setAlignment(Qt.AlignJustify)
        )
        self.format_toolbar.addAction(self.parent().alignj_action)
        format_menu.addAction(self.parent().alignj_action)

        format_group = QActionGroup(self.parent())
        format_group.setExclusive(True)
        format_group.addAction(self.parent().alignl_action)
        format_group.addAction(self.parent().alignc_action)
        format_group.addAction(self.parent().alignr_action)
        format_group.addAction(self.parent().alignj_action)

        format_menu.addSeparator()

        # A list of all format-related widgets/actions, so we can disable/enable signals when updating.
        self.parent()._format_actions = [
            self.parent().fonts,
            self.parent().fontsize,
            self.parent().bold_action,
            self.parent().italic_action,
            self.parent().underline_action,
            # We don't need to disable signals for alignment, as they are paragraph-wide.
        ]

        # At the end of your __init__ method
        self.set_scaled_icons()

        view_menu = self.parent().menuBar().addMenu("&View")

        reset_view_action = QAction(
            QIcon(""),
            "Reset View",
            self.parent(),
        )
        reset_view_action.setStatusTip("Reset View")
        reset_view_action.triggered.connect(
            lambda: (self.parent().createDock(), self.reset_toolbar_positions())
        )
        view_menu.addAction(reset_view_action)

        self.parent().editor.cursorPositionChanged.connect(self.update_format)

    def update_size(self, s: float):
        editor = self.parent().editor
        sizef = float(s)

        # Save current cursor/selection to restore later
        cur = editor.textCursor()
        had_selection = cur.hasSelection()
        sel_start = cur.selectionStart()
        sel_end = cur.selectionEnd()
        pos = cur.position()

        # Apply the size to the whole document
        doc_cursor = QTextCursor(editor.document())
        doc_cursor.beginEditBlock()
        doc_cursor.select(QTextCursor.Document)
        fmt = QTextCharFormat()
        fmt.setFontPointSize(sizef)
        doc_cursor.mergeCharFormat(fmt)
        doc_cursor.endEditBlock()

        # Update document default font so new text also matches
        default_font = editor.currentFont()
        default_font.setPointSize(int(sizef))
        editor.document().setDefaultFont(default_font)

        # Ensure typing uses the new size
        curr_fmt = editor.currentCharFormat()
        curr_fmt.setFontPointSize(sizef)
        editor.setCurrentCharFormat(curr_fmt)

        # Restore original cursor/selection
        new_cursor = editor.textCursor()
        if had_selection:
            new_cursor.setPosition(sel_start)
            new_cursor.setPosition(sel_end, QTextCursor.KeepAnchor)
        else:
            new_cursor.setPosition(pos)
        editor.setTextCursor(new_cursor)

    def block_signals(self, objects, b):
        for o in objects:
            if hasattr(o, "blockSignals"):
                o.blockSignals(b)

    def update_format(self):
        """Update the formatting toolbar/actions when the cursor position changes"""
        # Disable signals to avoid triggering format changes while updating UI
        self.block_signals(self.parent()._format_actions, True)

        # Get the current format at cursor position
        cursor = self.parent().editor.textCursor()
        char_format = cursor.charFormat()

        # Update font selector
        current_font = char_format.font()
        self.parent().fonts.setCurrentFont(current_font)

        # Update font size
        font_size = char_format.fontPointSize()
        if font_size > 0:  # fontPointSize returns 0 if not set
            size_index = -1
            for i, size in enumerate(FONT_SIZES):
                if size == int(font_size):
                    size_index = i
                    break
            if size_index >= 0:
                self.parent().fontsize.setCurrentIndex(size_index)

        # Update bold button
        self.parent().bold_action.setChecked(current_font.weight() >= QFont.Bold)

        # Update italic button
        self.parent().italic_action.setChecked(current_font.italic())

        # Update underline button
        self.parent().underline_action.setChecked(current_font.underline())

        # Update alignment buttons (paragraph-wide formatting)
        block_format = cursor.blockFormat()
        alignment = block_format.alignment()

        self.parent().alignl_action.setChecked(alignment == Qt.AlignLeft)
        self.parent().alignc_action.setChecked(alignment == Qt.AlignCenter)
        self.parent().alignr_action.setChecked(alignment == Qt.AlignRight)
        self.parent().alignj_action.setChecked(alignment == Qt.AlignJustify)

        # Re-enable signals
        self.block_signals(self.parent()._format_actions, False)

    # Add this function to your Toolbar class
    def set_scaled_icons(self):
        # Enable icon scaling
        for toolbar in [
            self.actions(),
            self.edit_toolbar.actions(),
            self.format_toolbar.actions(),
        ]:
            for action in toolbar:
                if action.icon().isNull():
                    continue

                # Get the original icon
                icon = action.icon()
                pixmap = icon.pixmap(QSize(16, 16))

                # Create a scaled version
                scaled_pixmap = pixmap.scaled(
                    QSize(24, 24),  # Target size
                    Qt.KeepAspectRatio,
                    Qt.SmoothTransformation,
                )

                # Set the scaled icon
                action.setIcon(QIcon(scaled_pixmap))



from rephraser.lib.DarkPallete import DarkPalette
from rephraser.RePhraser import MainWindow
from PyQt5.QtWidgets import QApplication, QStyleFactory

# from PyQt5.QtGui import *

import sys
import os

from rephraser import basedir
from PyQt5.QtGui import QIcon

if __name__ == "__main__":

    # try:
    app = QApplication(sys.argv)

    fusion = QStyleFactory.create("Fusion")
    QApplication.setStyle(fusion)

    # Now use a palette to switch to dark colors:
    dark_palette = DarkPalette()
    QApplication.setPalette(dark_palette)

    # qdarktheme.enable_hi_dpi()
    # qdarktheme.setup_theme(
    #     custom_colors={"primary": "#D0BCFF", "background": "24273a", "statusBar.background": "24273a", "toolbar.background": "24273a", "background>title": "c5c2c5", "foreground": "c5cff5", "border": "#39394a"})
    # qdarktheme.stop_sync()

    app.setStyleSheet("".join(open(basedir / "dark.qss").readlines()))
    ico = basedir / "Rephraser.ico"
    app.setWindowIcon(QIcon(str(ico)))

    # window = PasteFromAuthorDialog()
    window = MainWindow()
    window.setWindowIcon(QIcon(str(ico)))
    # window = QMainWindow()
    # lbl = QLabel("Test")
    # window.setCentralWidget(lbl)
    window.show()

    # customTitlebarWindow = CustomTitlebarWindow(window)
    # customTitlebarWindow.setTopTitleBar(icon_filename='dark-notepad.svg')
    # # customTitlebarWindow.setButtonHint(['close'])
    # customTitlebarWindow.setButtons()
    # customTitlebarWindow.show()

    app.exec_()
# except Exception as e:
#     print(traceback.format_exc())



from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtPrintSupport import *

import os
import sys

# import math

# import traceback

from ..lib.ClickableLabel import ClickableLabel
from ..lib.AuthorEntry import AuthorEntry
from ..lib.Stores import store


class AddAuthorDialog(QDialog):

    submit = pyqtSignal(AuthorEntry)

    def __init__(self, entry, parent=None):
        super(QDialog, self).__init__(parent=parent)

        print("TST")
        p = entry.getProperties(include_name=True)
        print(p)

        author_name = p["author_name"]
        foreground = p["foreground"]
        background = p["background"]
        italic = p["italic"]
        weight = p["weight"]
        href = p["href"]

        self.original_row = {
            "author_name": author_name,
            "foreground": foreground,
            "background": background,
            "italic": italic,
            "weight": weight,
            "href": href,
        }

        mainlayout = QVBoxLayout()

        author_cont = QHBoxLayout()
        author_cont.addWidget(QLabel("Author Name: "))
        self.author_field = QLineEdit(author_name)
        author_cont.addWidget(self.author_field)

        foreground_cont = QHBoxLayout()
        foreground_cont.addWidget(QLabel("Foreground: "))
        self.colorPrev_foreground = ClickableLabel(QColor(foreground))
        foreground_cont.addWidget(self.colorPrev_foreground)

        background_cont = QHBoxLayout()
        background_cont.addWidget(QLabel("Background: "))
        self.colorPrev_background = ClickableLabel(QColor(background))
        background_cont.addWidget(self.colorPrev_background)

        weight_cont = QHBoxLayout()
        weight_cont.addWidget(QLabel("Weight: "))
        self.weight_spinbox = QSpinBox()
        self.weight_spinbox.setMinimum(0)
        self.weight_spinbox.setMaximum(100)
        self.weight_spinbox.setSingleStep(1000)
        self.weight_spinbox.setValue(weight)
        weight_cont.addWidget(self.weight_spinbox)

        formatting_cont = QHBoxLayout()
        self.isItalic = QCheckBox("Italic")
        self.isItalic.setChecked(italic)
        # self.isBold = QCheckBox("Bold")
        # formatting_cont.addWidget(self.isBold)
        formatting_cont.addWidget(self.isItalic)

        href_cont = QHBoxLayout()
        href_cont.addWidget(QLabel("Href: "))
        self.href_field = QLineEdit()
        self.href_field.setText(href)
        href_cont.addWidget(self.href_field)

        self.saveAuthor_btn = QPushButton("Save Author")

        # close before adding lambda function
        self.saveAuthor_btn.clicked.connect(lambda: self.fin())

        mainlayout.addLayout(author_cont)
        mainlayout.addLayout(foreground_cont)
        mainlayout.addLayout(background_cont)
        mainlayout.addLayout(weight_cont)
        mainlayout.addLayout(formatting_cont)
        mainlayout.addLayout(href_cont)
        mainlayout.addWidget(self.saveAuthor_btn)

        self.setLayout(mainlayout)

    def fin(self):
        author_name = self.author_field.text()
        color = self.colorPrev_foreground.color.name(QColor.HexArgb)
        background = self.colorPrev_background.color.name(QColor.HexArgb)
        weight = self.weight_spinbox.value()
        italic = self.isItalic.isChecked()
        href = self.href_field.text()
        # rowNum = self.selectedRows()[0] if overrideRow else None

        # print(self.parent().author_dictionary)
        if self.original_row["author_name"] == author_name:
            # means the override was meant
            print("OVERRIDE MODE")
            pass

        self.done(QDialog.Accepted)

        self.submit.emit(
            AuthorEntry(author_name, color, background, weight, italic, href)
        )


class AuthorTable(QTableWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.parent_ = parent

        self.settings = QSettings("DeadBush225", "RePhraser")

        # store.author_dictionary = {}
        store.author_dictionary = self.settings.value("authors")
        # print(retrieved)

        self.setColumnCount(1)
        self.setHorizontalHeaderLabels(["Author Signature"])
        self.verticalHeader().hide()
        # store.author_dictionary = author_dictionary

        if not store.author_dictionary:
            store.author_dictionary = {
                "Ai": {
                    "italic": False,
                    "weight": 0,
                    "foreground": QColor("#fc3737"),
                    "background": QColor("#fc6f37"),
                    "href": "WWW",
                },
                "ChatGPT": {
                    "italic": False,
                    "weight": 100,
                    "foreground": QColor("#3772fc"),
                    "background": QColor("#53D85A"),
                    "href": "AAA",
                },
                "Rhixie": {
                    "italic": True,
                    "weight": 0,
                    "foreground": QColor("#fcdb37"),
                    "background": QColor("#9037fc"),
                    "href": "CCC",
                },
            }

        keys = store.author_dictionary.keys()
        for author_name in keys:
            if not author_name in store.author_dictionary:
                print(f"is '{author_name}' in {store.author_dictionary}?")
                continue

            # print("AUTHOR NAME")
            # print(store.author_dictionary[author_name])

            self.addEntry(
                AuthorEntry(author_name, **store.author_dictionary[author_name])
            )

        # self.author_table.resizeColumnsToContents()
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.SingleSelection)

        header = self.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.Stretch)

        self.cellDoubleClicked.connect(
            lambda row, col: self.addAuthor(
                author_name=self.cellWidget(row, 0).text(), row_count=row
            )
        )

        self.cellClicked.connect(self.table_selection_changed)

    def addAuthor(self, author_name="", row_count=None):
        print(f"ADDING AUTHOR: {author_name}")

        new_entry = AuthorEntry(author_name=author_name)

        if row_count and author_name:  # `author_name` is just for error handling
            new_entry.foreground = QColor(
                store.author_dictionary[author_name]["foreground"],
            )
            new_entry.background = QColor(
                store.author_dictionary[author_name]["background"],
            )
            new_entry.italic = store.author_dictionary[author_name]["italic"]
            new_entry.weight = store.author_dictionary[author_name]["weight"]
            new_entry.href = store.author_dictionary[author_name]["href"]

        self.addAuthorWidget = AddAuthorDialog(new_entry, parent=self)
        self.addAuthorWidget.submit.connect(
            lambda entry: self.addEntry(entry, row_count=row_count),
        )

        self.addAuthorWidget.exec_()

    def addEntry(self, entry, row_count=None):
        if row_count == None:
            row_count = self.rowCount()
            self.insertRow(row_count)

            store.author_dictionary[entry.author_name] = entry.getProperties()

        elif row_count != None:
            print("ROW DEFINED: OVERRIDING")

            # row is defined which means it is overriden
            # replace old name with new
            old_name = self.cellWidget(self.selectedRows()[0], 0).text()
            # store.author_dictionary = {
            #     (key if key != old_name else author_name): value
            #     for (key, value) in store.author_dictionary.items()
            # }
            store.author_dictionary[old_name] = entry.getProperties()

        self.saveSettings()
        # print(self.settings.value("authors"))

        textCell = QTableWidgetItem(entry.author_name)
        textCell.setFlags(Qt.ItemIsSelectable)

        # self.setItem(row_count, 0, textCell)

        signature_preview = QLabel(entry.author_name)
        signature_preview.setAlignment(Qt.AlignCenter)
        signature_preview.setStyleSheet(entry.getStyleSheet())

        self.setCellWidget(row_count, 0, signature_preview)

    def table_selection_changed(self, row, col):
        print(f"SELECTION CHANGED {row}")
        # selected_rows = {r.row() for r in self.selectedIndexes()}

        # if len(selected_rows) != 1:
        #     return

        # print("CHANGING AUTHOR")
        # selected_row = list(selected_rows)[0]
        selected_row = row

        author_name = self.cellWidget(selected_row, 0).text()

        self.parent_.editor.setTextCharFormat(author_name)
        self.parent_.editor.setCharFormatSelection()
        self.parent_.editor.setFocus()

    def mousePressEvent(self, e):
        super().mousePressEvent(e)

        if e.button() != Qt.RightButton:
            return

        print("MOUSE RIGHT CLICK")

        selected_rows = self.selectedRows()
        # selected_rows.sort()

        if len(selected_rows) == 1:
            selected_row = selected_rows[0]
            print("TRYING TO DELETE")
            name = self.item(selected_row, 0).text()
            print(name)
            print(store.author_dictionary)

            store.author_dictionary.pop(name)
            self.removeRow(selected_row)

            self.saveSettings()

    def saveSettings(self):
        self.settings.setValue("authors", store.author_dictionary)

    def selectedRows(self):
        return list(set([r.row() for r in self.selectedIndexes()]))



from PyQt5.QtGui import QPalette, QColor
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget
import sys

from rephraser.lib.Logger import Logger

class DarkPalette(QPalette):
    def __init__(self):
        super().__init__()
        self.setColor(QPalette.Window, QColor(53, 53, 53))
        self.setColor(QPalette.WindowText, Qt.white)
        self.setColor(QPalette.Base, QColor(35, 35, 35))
        self.setColor(QPalette.AlternateBase, QColor(53, 53, 53))
        self.setColor(QPalette.ToolTipBase, QColor(25, 25, 25))
        self.setColor(QPalette.ToolTipText, Qt.white)
        self.setColor(QPalette.Text, Qt.white)
        self.setColor(QPalette.Button, QColor(53, 53, 53))
        self.setColor(QPalette.ButtonText, Qt.white)
        self.setColor(QPalette.BrightText, Qt.red)
        self.setColor(QPalette.Link, QColor(42, 130, 218))
        self.setColor(QPalette.Highlight, QColor(42, 130, 218))
        self.setColor(QPalette.HighlightedText, QColor(35, 35, 35))
        self.setColor(QPalette.Active, QPalette.Button, QColor(53, 53, 53))
        self.setColor(QPalette.Disabled, QPalette.ButtonText, Qt.darkGray)
        self.setColor(QPalette.Disabled, QPalette.WindowText, Qt.darkGray)
        self.setColor(QPalette.Disabled, QPalette.Text, Qt.darkGray)
        self.setColor(QPalette.Disabled, QPalette.Light, QColor(53, 53, 53))

def enable_dark_titlebar( window: QWidget):
    """Enable dark titlebar on Windows"""
    if sys.platform == 'win32':
        try:
            import ctypes
            from ctypes.wintypes import DWORD, BOOL, HRGN, HWND
            
            # Detect Windows version to use correct constant
            win_version = sys.getwindowsversion()
            if (win_version.major == 10 and win_version.build >= 17763) or win_version.major > 10:
                DWMWA_USE_IMMERSIVE_DARK_MODE = 20  # Windows 10 1809+
            else:
                DWMWA_USE_IMMERSIVE_DARK_MODE = 19  # Earlier Windows 10
            
            hwnd = HWND(window.winId().__int__())
            value = BOOL(True)
            
            ctypes.windll.dwmapi.DwmSetWindowAttribute(
                hwnd, 
                DWMWA_USE_IMMERSIVE_DARK_MODE,
                ctypes.byref(value),
                ctypes.sizeof(value)
            )
            
            Logger.w("Dark titlebar enabled successfully", Logger.INFO)
        except Exception as e:
            Logger.w("Failed to set dark titlebar: {e}", Logger.WARNING)